#include "app.h"

#define STAR_MAX	64

#define NOISE_X		((128 - 16 - 16) / 4)
#define NOISE_Y		(  88            / 4)
#define NOISE_TBL	(NOISE_X * NOISE_Y)

static void TSKframeExit(TSK *tskp);
static void TSKframeDraw(TSK *tskp);
static void TSKmaskExit(TSK *tskp);
static void TSKmaskDraw(TSK *tskp);
static void TSKnoiseExit(TSK *tskp);
static void TSKnoiseDraw(TSK *tskp);
static void TSKalertExit(TSK *tskp);
static void TSKalertDraw(TSK *tskp);
static void TSKstarExit(TSK *tskp);
static void TSKstarDraw(TSK *tskp);

static fixed star_pos[STAR_MAX][XY];
static fixed star_vel[STAR_MAX];

static int noise_tbl[NOISE_TBL];

void TSKframe(TSK *tskp)
{
	switch(tskp->step){
		case	0:
			tskp->exit_adr = &TSKframeExit;
			tskp->draw_adr = &TSKframeDraw;
			tskp->step++;
			break;
		case	1:
			break;
		default:
			TSKclear(tskp);
			break;
	}
}

static void TSKframeExit(TSK *tskp)
{
}

static void TSKframeDraw(TSK *tskp)
{
	int w = MapX( 80);
	int h = MapY(480);
	render_rectangle_fill(&render,          0, 0, w, h, 3);
	render_rectangle_fill(&render, DISP_X - w, 0, w, h, 3);
}

void TSKmask(TSK *tskp)
{
	switch(tskp->step){
		case	0:
			tskp->exit_adr = &TSKmaskExit;
			tskp->draw_adr = &TSKmaskDraw;
			tskp->step++;
			break;
		case	1:
			break;
		default:
			TSKclear(tskp);
			break;
	}
}

static void TSKmaskExit(TSK *tskp)
{
}

static void TSKmaskDraw(TSK *tskp)
{
	int w = MapX(80);
	int h = MapY(32);
	render_rectangle_fill(&render, w,          0, DISP_X - w * 2, h, 3);
	render_rectangle_fill(&render, w, DISP_Y - h, DISP_X - w * 2, h, 3);
}

void TSKnoise(TSK *tskp)
{
	int i;

	switch(tskp->step){
		case	0:
			tskp->exit_adr = &TSKnoiseExit;
			tskp->draw_adr = &TSKnoiseDraw;
			tskp->cnt = 0;
			for(i = 0; i < NOISE_TBL; i++){
				noise_tbl[i] = i / 36;
			}
			TBLshuffle(noise_tbl, NOISE_TBL);
			tskp->wait = 90;
			tskp->step++;
			break;
		case	1:
			if(tskp->wait > 75){
				tskp->cnt++;
				tskp->wait--;
			}else{
				noise_flag = 1;
				tskp->step++;
			}
			break;
		case	2:
			if(tskp->wait > 15){
				tskp->wait--;
			}else{
				tskp->step++;
			}
			break;
		case	3:
			if(tskp->wait){
				tskp->cnt--;
				tskp->wait--;
			}else{
				tskp->step = -1;
			}
			break;
		default:
			TSKclear(tskp);
			break;
	}
}

static void TSKnoiseExit(TSK *tskp)
{
}

static void TSKnoiseDraw(TSK *tskp)
{
	static int save_c = 1; /* ̂߁Arnd32()Ɏ֐Ɏ */
	int c = save_c;        /* ̂߁Arnd32()Ɏ֐Ɏ */
	//
	int i;
	int x1;
	int y1;
	int x2;
	int y2;
	unsigned char* p;

	i = 0;
	for(y1 = 0; y1 < 88; y1 += 4) {
		for(x1 = (0 + 16); x1 < (128 - 16); x1 += 4) {
			if(noise_tbl[i++] < tskp->cnt) {
				p = &vbuff[y1 * DISP_X + x1];
				for(y2 = 0; y2 < 4; y2++) {
					for(x2 = 0; x2 < 4; x2++) {
						c ^= (c << 13); /* ̂߁Arnd32()Ɏ֐Ɏ */
						c ^= (c >> 17); /* ̂߁Arnd32()Ɏ֐Ɏ */
						c ^= (c <<  5); /* ̂߁Arnd32()Ɏ֐Ɏ */
						*p++ = c & 3;
					}
					p += (128 - 4);
				}
			}
		}
	}

	save_c = c; /* ̂߁Arnd32()Ɏ֐Ɏ */
}

static void set_alert_color(int c)
{
	static const unsigned short ctbl[5][16] = {
		{0,0,0,0,0,1,1,1,1,1,2,2,2,2,2,3},
		{0,0,1,1,1,1,1,2,2,2,2,2,3,3,3,3},
		{0,0,0,0,1,1,1,1,1,2,2,2,2,2,3,3},
		{0,1,1,1,1,1,2,2,2,2,2,3,3,3,3,3},
		{0,0,0,1,1,1,1,1,2,2,2,2,2,3,3,3},
	};

	c -= 16;	/* 16`63 -> 0`47 */
	c /=  8;	/*  0`47 -> 0` 5 */
	if(c <  0) c =  0;
	if(c > 15) c = 15;
	fill_color = ctbl[now % 5][c] ^ 3;
}

void TSKalert(TSK *tskp)
{
	switch(tskp->step){
		case	0:
			tskp->exit_adr = &TSKalertExit;
			tskp->draw_adr = &TSKalertDraw;
			tskp->color.r = 16;
			set_alert_color(tskp->color.r);
			SDL_PCB_SndStopSe(SND_ALERT);
			tskp->step++;
			break;
		case	1:
			if(!limit_time){
				tskp->wait = 48;
				tskp->step++;
			}
			break;
		case	2:
			if(!SDL_PCB_SndIsPlaySe(7)){
				SDL_PCB_SndPlaySe(SND_ALERT);
			}
			if(limit_time){
				tskp->step = 0;
				break;
			}
			tskp->color.r++;
			if(tskp->wait){
				tskp->wait--;
			}else{
				tskp->wait = 48;
				tskp->step++;
			}
			set_alert_color(tskp->color.r);
			break;
		case	3:
			if(!SDL_PCB_SndIsPlaySe(7)){
				SDL_PCB_SndPlaySe(SND_ALERT);
			}
			if(limit_time){
				tskp->step = 0;
				break;
			}
			tskp->color.r--;
			if(tskp->wait){
				tskp->wait--;
			}else{
				tskp->wait = 48;
				tskp->step--;
			}
			set_alert_color(tskp->color.r);
			break;

		default:
			TSKclear(tskp);
			break;
	}
}

static void TSKalertExit(TSK *tskp)
{
}

static void TSKalertDraw(TSK *tskp)
{
}

void TSKstar(TSK *tskp)
{
	int i;

	switch(tskp->step){
		case	0:
			tskp->exit_adr = &TSKstarExit;
			tskp->draw_adr = &TSKstarDraw;
			for(i = 0; i < STAR_MAX; i++){
				star_pos[i][X] = FRND_RANGE(seed, GRP_MIN_X, GRP_MAX_X);
				star_pos[i][Y] = FRND_RANGE(seed, GRP_MIN_Y, GRP_MAX_Y);
				star_vel[i]    = ffabs(frand());
			}
			tskp->step++;
			break;
		case	1:
			for(i = 0; i < STAR_MAX; i++) {
				if(scr_vel[X]) {
					fadd_(star_pos[i][X], scr_vel[X]);
					if(ficom(scr_vel[X], <, 0)) { fsub_(star_pos[i][X], star_vel[i]); if(fcom(star_pos[i][X], < , GRP_MIN_X)) fadd_(star_pos[i][X], GRP_SIZE_X); }
					                       else { fadd_(star_pos[i][X], star_vel[i]); if(fcom(star_pos[i][X], >=, GRP_MAX_X)) fsub_(star_pos[i][X], GRP_SIZE_X); }
				}
				if(scr_vel[Y]) {
					fadd_(star_pos[i][Y], scr_vel[Y]);
					if(ficom(scr_vel[Y], <, 0)) { fsub_(star_pos[i][Y], star_vel[i]); if(fcom(star_pos[i][Y], < , GRP_MIN_Y)) fadd_(star_pos[i][Y], GRP_SIZE_Y); }
					                       else { fadd_(star_pos[i][Y], star_vel[i]); if(fcom(star_pos[i][Y], >=, GRP_MAX_Y)) fsub_(star_pos[i][Y], GRP_SIZE_Y); }
				}
			}
			break;

		default:
			TSKclear(tskp);
			break;
	}
}

static void TSKstarExit(TSK *tskp)
{
}

static void TSKstarDraw(TSK *tskp)
{
	static int seed;
	//
	int i;
	int c;

	for(i = 0; i < STAR_MAX; i++){
		c = RND32(seed) & 3;
		pointSDLsurface(fist(star_pos[i][X]), fist(star_pos[i][Y]), c);
	}
}

