#include "app.h"

#define RANKING_FNAME	"ranking.dat"

static void TSKrankingExit(TSK *tskp);
static void TSKrankingDraw(TSK *tskp);
static void setRankingGrp(TSK* tskp);
static void TSKrankingSetMode(TSK *tskp);

static char *rank_str[] = {
							" 1ST",
							" 2ND",
							" 3RD",
							" 4TH",
							" 5TH",
							" 6TH",
							" 7TH",
							" 8TH",
							" 9TH",
							"10TH",
						};

void TSKranking(TSK *tskp)
{
	switch(tskp->step){
		case	0:
			tskp->exit_adr = &TSKrankingExit;
			tskp->draw_adr = &TSKrankingDraw;
			TSKrankingSetMode(tskp);
			tskp->wait = 60 * 15;
			tskp->cnt = 0;
			break;
		case	1:
			tskp->pos[X] = tskp->trg[X] * (tskp->rate) + tskp->vel[X] * (1.0f - tskp->rate);	/* ݈ʒu = ڕWʒu*rate + Oʒu*(1-rate) */
			tskp->pos[Y] = tskp->trg[Y] * (tskp->rate) + tskp->vel[Y] * (1.0f - tskp->rate);
			tskp->rate += (1.0f - tskp->rate) / 5.0f + 0.01f/**/;
			if(tskp->rate > 1.0f) tskp->rate = 1.0f;
			tskp->cnt++;
			if(tskp->wait){
				tskp->wait--;
			}else{
				tskp->step = -1;
			}
			break;
		case	2:
			tskp->scl[X] = tskp->trg[X] * (tskp->rate) + tskp->vel[X] * (1.0f - tskp->rate);	/* ]TCY = ڕWTCY*rate + OTCY*(1-rate) */
			tskp->scl[Y] = tskp->trg[Y] * (tskp->rate) + tskp->vel[Y] * (1.0f - tskp->rate);
			tskp->rate += 1.0f / 15.0f;
			if(tskp->rate > 1.0f) tskp->rate = 1.0f;
			tskp->cnt++;
			if(tskp->wait){
				tskp->wait--;
			}else{
				tskp->step = -1;
			}
			break;
		default:
			TSKclear(tskp);
			break;
	}
}

static void TSKrankingExit(TSK *tskp)
{
}

static void clipSDLmask(fixed px, fixed py, fixed w, fixed h)
{
	int x0;
	int y0;
	int x1;
	int y1;

	fsub_(px, fidiv(w, 2));
	fsub_(py, fidiv(h, 2));

	px = MapX(px);
	py = MapY(py);
	 w = MapX( w);
	 h = MapY( h);

	fadd_(w, fld(0.5)); /*  */
	fadd_(h, fld(0.5)); /*  */

	x0 = fist(px);
	y0 = fist(py);
	x1 = x0 + fist(w);
	y1 = y0 + fist(h);

	if(x0 <      0) x0 =      0;
	if(y0 <      0) y0 =      0;
	if(x1 > DISP_X) x1 = DISP_X;
	if(y1 > DISP_Y) y1 = DISP_Y;

	render.left   = x0;
	render.top    = y0;
	render.right  = x1;
	render.bottom = y1;

	if(now & 1) {
		render_rectangle_dark(&render, 0, 0, DISP_X, DISP_Y, -1, 0);
	}
}

static void TSKrankingDraw(TSK *tskp)
{
	clipSDLmask(fld(tskp->pos[X]), fld(tskp->pos[Y]), fld(tskp->scl[X]), fld(tskp->scl[Y]));
	/*NbsO*/
	setRankingGrp(tskp);
	/*NbsO*/
	render.left   =      0;
	render.top    =      0;
	render.right  = DISP_X;
	render.bottom = DISP_Y;
}

void ranking_init(void)
{
	int i;

	for(i = 0; i < RANK_MAX; i++){
		ranking[i].score = 1000 * (RANK_MAX - i);
		ranking[i].stage = RANK_MAX - i - 1;
		ranking[i].playtime = 60 * 60 * 60 * 100 - 1;
		ranking[i].avgtime = 60 * 30;
	}
}

int ranking_save(void)
{
	int result = 0;
	FILEACC fa;

	if(pceFileCreate(RANKING_FNAME, sizeof ranking) != 0) {
		fprintf(stderr, "save : ranking data create error.\n");
	} else {
		if(pceFileOpen(&fa, RANKING_FNAME, FOMD_WR) != 0) {
			fprintf(stderr, "save : ranking data open error.\n");
		} else {
			if(pceFileWriteSct(&fa, ranking, 0, sizeof ranking) != sizeof ranking) {
				fprintf(stderr, "save : ranking data write error.\n");
			} else {
				result = 1;
			}
			pceFileClose(&fa);
		}
	}

	return result;
}

int ranking_load(void)
{
	int result = 0;
	FILEACC fa;

	if(pceFileOpen(&fa, RANKING_FNAME, FOMD_RD) != 0) {
		fprintf(stderr, "load : ranking data open error.\n");
	} else {
		if(pceFileReadSct(&fa, ranking, 0, sizeof ranking) != sizeof ranking) {
			fprintf(stderr, "load : ranking data read error.\n");
		} else {
			result = 1;
		}
		pceFileClose(&fa);
	}

	return result;
}

int ranking_set(void)
{
	int i, j;

	for(i = 0; i < RANK_MAX; i++){
		if(score < ranking[i].score) continue;
		if(score == ranking[i].score){
			if(ranking[i].stage >= stage) continue;
			if(ranking[i].playtime >= playtime) continue;
			if(ranking[i].avgtime >= (playtime / stage)) continue;
		}
		for(j = RANK_MAX - 1; j > i; j--){
			ranking[j] = ranking[j-1];
		}
		ranking[i].score    = score;
		ranking[i].stage    = stage;
		ranking[i].playtime = playtime;
		ranking[i].avgtime  = playtime / stage;
		return i;
	}

	return -1;
}

static void setRankingGrp(TSK* tskp)
{
	char str[256];
	int i;
	int time, m, s;
	float x, y;

	y = tskp->pos[Y] - (asc_size_y * 6) + (asc_size_y / 2.0f);

	strcpy(str, "- TOP 10 RANKING -");
	x = tskp->pos[X] - fst(getWidthASCII(str)) / 2.0f + (asc_size_x / 2.0f);
	drawASCII(str, fld(x), fld(y));
	y += asc_size_y;

	strcpy(str, "RANK SCORE STAGE TIME ");
	x  = tskp->pos[X] - fst(getWidthASCII(str)) / 2.0f + (asc_size_x / 2.0f);
	drawASCII(str, fld(x), fld(y));
	y += asc_size_y;

	strcpy(str, "10TH 99999999 99 99:99");
	x  = tskp->pos[X] - fst(getWidthASCII(str)) / 2.0f + (asc_size_x / 2.0f);

	for(i = 0; i < RANK_MAX; i++){
		time = ranking[i].playtime;
		m    = (time / (60 * 60)) % 100;
		s    = (time /  60      ) %  60;
		sprintf(str, "%s %08d %02d %02d:%02d", rank_str[i], ranking[i].score, ranking[i].stage + 1, m, s);
		if((i != tskp->mode) || !(now & 0x20)) {
			drawASCII(str, fld(x), fld(y));
		}
		y += asc_size_y;
	}
}

static void TSKrankingSetMode(TSK *tskp)
{
	int ret = rand() % 4;

	switch(ret){
		case	0:
			tskp->pos[X] = 320.0f;			/* ݈ʒu */
			tskp->pos[Y] = -240.0f;
			tskp->vel[X] = tskp->pos[X];	/* Oʒu */
			tskp->vel[Y] = tskp->pos[Y];
			tskp->trg[X] = 320.0f;			/* ڕWʒu */
			tskp->trg[Y] = 240.0f;
			tskp->scl[X] = 450.0f;			/* ]TCY */
			tskp->scl[Y] = 400.0f;
			tskp->rate = 0.0f;
			tskp->step = 1;
			break;
		case	1:
			tskp->pos[X] = 320.0f;			/* Œʒu */
			tskp->pos[Y] = 240.0f;
			tskp->scl[X] = 450.0f;			/* ݃TCY */
			tskp->scl[Y] = 200.0f;
			tskp->vel[X] = tskp->scl[X];	/* OTCY */
			tskp->vel[Y] = tskp->scl[Y];
			tskp->trg[X] = 450.0f;			/* ڕWTCY */
			tskp->trg[Y] = 400.0f;
			tskp->rate = 0.0f;
			tskp->step = 2;
			break;
		case	2:
			tskp->pos[X] = 320.0f;			/* Œʒu */
			tskp->pos[Y] = 240.0f;
			tskp->scl[X] = 225.0f;			/* ݃TCY */
			tskp->scl[Y] = 400.0f;
			tskp->vel[X] = tskp->scl[X];	/* OTCY */
			tskp->vel[Y] = tskp->scl[Y];
			tskp->trg[X] = 450.0f;			/* ڕWTCY */
			tskp->trg[Y] = 400.0f;
			tskp->rate = 0.0f;
			tskp->step = 2;
			break;
		case	3:
			tskp->pos[X] = 320.0f;			/* Œʒu */
			tskp->pos[Y] = 240.0f;			
			tskp->scl[X] = 225.0f;			/* ݃TCY */
			tskp->scl[Y] = 200.0f;			
			tskp->vel[X] = tskp->scl[X];	/* OTCY */
			tskp->vel[Y] = tskp->scl[Y];	
			tskp->trg[X] = 450.0f;			/* ڕWTCY */
			tskp->trg[Y] = 400.0f;
			tskp->rate = 0.0f;
			tskp->step = 2;
			break;
		default:
			DIE();
	}
}
