#include "app.h"

#define ACC_MAX (6.0f)
#define ACC_RATE (0.08f)
#define DIR_BLANK (4)

#define EXTEND_1ST (50000)
#define EXTEND_2ND (100000)
#define EXTEND_3RD (200000)

#define TIME_OFF (0)

#if !TIME_OFF
#define STG_LIMIT_TIME (60 * 30)
#else
#define STG_LIMIT_TIME (60 * 0)
#endif

static void TSKshipInt(TSK *tskp);
static void TSKshipExit(TSK *tskp);
static void TSKshipDraw(TSK *tskp);
static void TSKshipAccCtrl(TSK *tskp);
static void TSKshipShotCtrl(TSK *tskp);
static void TSKshipExtend(TSK *tskp);
static void TSKshipTimeCtrl(TSK *tskp);
static void TSKshipAttract(TSK *tskp);
static void ship_destroy(TSK *tskp);

static VERTEX vertex[] = {
							{ fld( +0.0), fld(-12.0) },
							{ fld(-12.0), fld(+12.0) },
							{ fld( +0.0), fld( +3.0) },
							{ fld(+12.0), fld(+12.0) },
						};

static float ship_vel[][XYZ] = {
								{ +0.0f, +0.0f,      +0.0f },	/* 0x00 (none) */
								{ +2.0f, +0.0f, +PI / 2.0f },	/* 0x01 PAD_RI */
								{ -2.0f, +0.0f, -PI / 2.0f },	/* 0x02 PAD_LF */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x03 (none) */
								{ +0.0f, +2.0f, +PI * 1.0f },	/* 0x04 PAD_DN */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x05 (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x06 (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x07 (none) */
								{ +0.0f, -2.0f, +PI * 0.0f },	/* 0x08 PAD_UP */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x09 (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x0A (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x0B (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x0C (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x0D (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x0E (none) */
								{ +0.0f, +0.0f,      +0.0f },	/* 0x0F (none) */
							};

static int ship_pads;
static int ship_dirm;
static int extend_step;
static int extend_score;
static int shot_wait;
static float shot_len;

static int time_off = TIME_OFF;

void TSKship(TSK *tskp)
{
	if(!attract_mode){
		ship_pads = pcb_pads;
	}else{
		TSKshipAttract(tskp);
	}

	switch(tskp->step){
		case	0:
			tskp->tskid |= TSKID_SHIP;
			tsk_ship = tskp;
			tskp->int_adr = NULL;
			tskp->exit_adr = &TSKshipExit;
			tskp->draw_adr = NULL;
			tskp->col[X] = 4.0f;
			tskp->col[Y] = 4.0f;
			tskp->vel[X] = 2.0f;
			tskp->vel[Y] = 2.0f;
			setGRPlineObj(tskp, vertex, ARRAY_SIZE(vertex));
			TSKpalleteSet(tskp, 255, 255, 255, 255);
			extend_step = 0;
			dead_flag = 0;
			limit_time = STG_LIMIT_TIME;
			break;
		case	1:
			tskp->draw_adr = &TSKshipDraw;
			SDL_PCB_SndPlaySe(SND_SHIP);
			tskp->pos[X] = SCREEN_X / 2.0f;
			tskp->pos[Y] = SCREEN_Y / 2.0f;
			tskp->acc[X] = 0.0f;
			tskp->acc[Y] = 0.0f;
			tskp->wait = 60;
			tskp->trg_tsk = NULL;
			tskp->old_tsk = NULL;
			ship_dirm = PAD_UP;
			tskp->roll = ship_vel[ship_dirm][Z];
			shot_wait = 0;
			shot_len = SSHOT_LENGTH;
			tskp->step++;
		case	2:
			TSKshipTimeCtrl(tskp);
			if(tskp->int_adr == NULL){
				TSKpalleteSet(tskp, (rand() % 255), (rand() % 255), (rand() % 255), 255);
				if(tskp->wait){
					tskp->wait--;
				}else{
					TSKpalleteSet(tskp, 255, 255, 255, 255);
					tskp->int_adr = &TSKshipInt;
				}
			}
			if(ship_pads & PAD_B){
				tskp->acc[X] /= 2.0f;
				tskp->acc[Y] /= 2.0f;
			}
			TSKshipAccCtrl(tskp);
			if((ship_pads & (PAD_UP|PAD_DN|PAD_LF|PAD_RI))){
				tskp->pos[X] += ship_vel[ship_dirm][X];
				tskp->pos[Y] += ship_vel[ship_dirm][Y];
				tskp->roll = ship_vel[ship_dirm][Z];
			}
			tskp->pos[X] += tskp->acc[X];
			tskp->pos[Y] += tskp->acc[Y];
			TSKareaClamp(tskp);
			TSKshipShotCtrl(tskp);
			break;
		case	3:
			if(tskp->wait){
				tskp->wait--;
			}else{
				limit_time = STG_LIMIT_TIME;
				tskp->draw_adr = NULL;
				dead_flag = 0;
				tskp->step++;
			}
			tskp->int_adr = NULL;
			tskp->trg_tsk = NULL;
			tskp->old_tsk = NULL;
			tskp->pos[X] += ship_vel[ship_dirm][X];
			tskp->pos[Y] += ship_vel[ship_dirm][Y];
			tskp->roll = ship_vel[ship_dirm][Z];
			tskp->pos[X] += tskp->acc[X];
			tskp->pos[Y] += tskp->acc[Y];
			TSKareaClamp(tskp);
			break;
		case	4:
			break;
		case	10:
			TSKshipTimeCtrl(tskp);
			if(tskp->acc[X] != 0.0f || tskp->acc[Y] != 0.0f){
				tskp->roll = ship_vel[ship_dirm][Z];
				tskp->pos[X] += tskp->acc[X];
				tskp->pos[Y] += tskp->acc[Y];
				if(tskp->acc[X] > +0.0f) tskp->acc[X] -= ACC_RATE;
				if(tskp->acc[X] < -0.0f) tskp->acc[X] += ACC_RATE;
				if(tskp->acc[Y] > +0.0f) tskp->acc[Y] -= ACC_RATE;
				if(tskp->acc[Y] < -0.0f) tskp->acc[Y] += ACC_RATE;
				if(fabs(tskp->acc[X]) <= ACC_RATE) tskp->acc[X] = 0.0f;
				if(fabs(tskp->acc[Y]) <= ACC_RATE) tskp->acc[Y] = 0.0f;
				TSKareaClamp(tskp);
			}else{
				tskp->trg_tsk = NULL;
				tskp->old_tsk = NULL;
				tskp->step = 2;
			}
			TSKshipShotCtrl(tskp);
			break;
		case	20:
			if(tskp->tskid & TSKID_DESTROY){
				tskp->draw_adr = NULL;
				SDL_PCB_SndPlaySe(SND_SDEST);
				TSKpalleteSet(tskp, 255, 255, 255, 255);
				setTSKbroken(tskp, 8, 4.0f);
				tskp->tskid &= ~TSKID_DESTROY;
				dead_flag = 1;
			}
			if(tskp->wait){
				tskp->wait--;
			}else{
				if(!time_off){
					limit_time += 60 * 10;
				}
				tskp->step = 1;
			}
			break;
		case	100:
			SDL_PCB_SndPlaySe(SND_SDEST);
			tskp->draw_adr = NULL;
			TSKpalleteSet(tskp, 255, 255, 255, 255);
			setTSKbroken(tskp, 8, 4.0f);
			game_mode = 2;
			tskp->step++;
		case	101:
			break;

		default:
			TSKclear(tskp);
			break;
	}

	TSKshipExtend(tskp);
}

static void TSKshipInt(TSK *tskp)
{
	if(tskp->tsk_collision->tskid & TSKID_ENEMY){
		if(tskp->old_tsk == tskp->tsk_collision) return;
		tskp->old_tsk = tskp->tsk_collision;
		if(ship_dirm & PAD_RI){
			ship_dirm = PAD_LF;
			tskp->acc[X]  = ship_vel[ship_dirm][X];
			tskp->acc[X] -= -tskp->acc[X];
			tskp->acc[Y]  = 0.0f;
		}else if(ship_dirm & PAD_LF){
			ship_dirm = PAD_RI;
			tskp->acc[X]  = ship_vel[ship_dirm][X];
			tskp->acc[X] -= -tskp->acc[X];
			tskp->acc[Y]  = 0.0f;
		}else if(ship_dirm & PAD_DN){
			ship_dirm = PAD_UP;
			tskp->acc[X]  = 0.0f;
			tskp->acc[Y]  = ship_vel[ship_dirm][Y];
			tskp->acc[Y] -= -tskp->acc[Y];
		}else if(ship_dirm & PAD_UP){
			ship_dirm = PAD_DN;
			tskp->acc[X]  = 0.0f;
			tskp->acc[Y]  = ship_vel[ship_dirm][Y];
			tskp->acc[Y] -= -tskp->acc[Y];
		}
		SDL_PCB_SndPlaySe(SND_REFLECT);
		tskp->step = 10;
		return;
	}
	ship_destroy(tskp);
}

static void TSKshipExit(TSK *tskp)
{
}

static void TSKshipDraw(TSK *tskp)
{
	drawGRPlineObj(tskp);
}

static void TSKshipAccCtrl(TSK *tskp)
{
	int dir;

	dir = (ship_pads & (PAD_UP|PAD_DN|PAD_LF|PAD_RI));
	if(dir){
		if(dir & PAD_RI)		dir = PAD_RI;
		else if(dir & PAD_LF)	dir = PAD_LF;
		else if(dir & PAD_DN)	dir = PAD_DN;
		else if(dir & PAD_UP)	dir = PAD_UP;

		if(dir & PAD_RI){
			if(ship_dirm & PAD_RI){
				tskp->acc[X] += (ACC_RATE / 2.0f);
				if(tskp->acc[X] > +ACC_MAX) tskp->acc[X] = +ACC_MAX;
			}else if(ship_dirm & PAD_LF){
				tskp->acc[X] = 0.0f;
			}else if(ship_dirm & PAD_DN){
				tskp->acc[X] = +fabs(tskp->acc[Y]);
				tskp->acc[Y] = 0.0f;
			}else if(ship_dirm & PAD_UP){
				tskp->acc[X] = +fabs(tskp->acc[Y]);
				tskp->acc[Y] = 0.0f;
			}
		}else if(dir & PAD_LF){
			if(ship_dirm & PAD_LF){
				tskp->acc[X] -= (ACC_RATE / 2.0f);
				if(tskp->acc[X] < -ACC_MAX) tskp->acc[X] = -ACC_MAX;
			}else if(ship_dirm & PAD_RI){
				tskp->acc[X] = 0.0f;
			}else if(ship_dirm & PAD_DN){
				tskp->acc[X] = -fabs(tskp->acc[Y]);
				tskp->acc[Y] = 0.0f;
			}else if(ship_dirm & PAD_UP){
				tskp->acc[X] = -fabs(tskp->acc[Y]);
				tskp->acc[Y] = 0.0f;
			}
		}else if(dir & PAD_DN){
			if(ship_dirm & PAD_DN){
				tskp->acc[Y] += (ACC_RATE / 2.0f);
				if(tskp->acc[Y] > +ACC_MAX) tskp->acc[Y] = +ACC_MAX;
			}else if(ship_dirm & PAD_UP){
				tskp->acc[Y] = 0.0f;
			}else if(ship_dirm & PAD_RI){
				tskp->acc[Y] = +fabs(tskp->acc[X]);
				tskp->acc[X] = 0.0f;
			}else if(ship_dirm & PAD_LF){
				tskp->acc[Y] = +fabs(tskp->acc[X]);
				tskp->acc[X] = 0.0f;
			}
		}else if(dir & PAD_UP){
			if(ship_dirm & PAD_UP){
				tskp->acc[Y] -= (ACC_RATE / 2.0f);
				if(tskp->acc[Y] < -ACC_MAX) tskp->acc[Y] = -ACC_MAX;
			}else if(ship_dirm & PAD_DN){
				tskp->acc[Y] = 0.0f;
			}else if(ship_dirm & PAD_RI){
				tskp->acc[Y] = -fabs(tskp->acc[X]);
				tskp->acc[X] = 0.0f;
			}else if(ship_dirm & PAD_LF){
				tskp->acc[Y] = -fabs(tskp->acc[X]);
				tskp->acc[X] = 0.0f;
			}
		}
		ship_dirm = dir;
	}else{
		if(tskp->acc[X] > +0.0f) tskp->acc[X] -= ACC_RATE;
		if(tskp->acc[X] < -0.0f) tskp->acc[X] += ACC_RATE;
		if(tskp->acc[Y] > +0.0f) tskp->acc[Y] -= ACC_RATE;
		if(tskp->acc[Y] < -0.0f) tskp->acc[Y] += ACC_RATE;
		if(fabs(tskp->acc[X]) <= ACC_RATE) tskp->acc[X] = 0.0f;
		if(fabs(tskp->acc[Y]) <= ACC_RATE) tskp->acc[Y] = 0.0f;
	}
}

static void TSKshipShotCtrl(TSK *tskp)
{
	TSK *etskp;
	float acc = fabs((tskp->acc[X] + tskp->acc[Y]));

	if(!shot_wait){
		if(ship_pads & PAD_A){
			etskp = TSKsetBtm(&TSKsshot, GROUP_SSHOT);
			if(etskp){
				etskp->roll = tskp->roll;
				etskp->len = SSHOT_LENGTH + 8.0f * acc;
				etskp->level = (int)(acc / ((ACC_MAX - 1.0f) / 4));
				if(ship_dirm & PAD_LF){
					etskp->pos[X] = tskp->pos[X] - 8.0f;
					etskp->pos[Y] = tskp->pos[Y];
					etskp->vel[X] = -SSHOT_SPD + tskp->acc[X] * 1.5f;
					etskp->vel[Y] = 0.0f;
				}else if(ship_dirm & PAD_RI){
					etskp->pos[X] = tskp->pos[X] + 8.0f;
					etskp->pos[Y] = tskp->pos[Y];
					etskp->vel[X] = +SSHOT_SPD + tskp->acc[X] * 1.5f;
					etskp->vel[Y] = 0.0f;
				}else if(ship_dirm & PAD_UP){
					etskp->pos[X] = tskp->pos[X];
					etskp->pos[Y] = tskp->pos[Y] - 8.0f;
					etskp->vel[X] = 0.0f;
					etskp->vel[Y] = -SSHOT_SPD + tskp->acc[Y] * 1.5f;
				}else if(ship_dirm & PAD_DN){
					etskp->pos[X] = tskp->pos[X];
					etskp->pos[Y] = tskp->pos[Y] + 8.0f;
					etskp->vel[X] = 0.0f;
					etskp->vel[Y] = +SSHOT_SPD + tskp->acc[Y] * 1.5f;
				}
				shot_wait = 10 - (int)acc;
			}
			TSKareaClamp(etskp);
		}
	}else{
		shot_wait--;
	}
}

static void TSKshipExtend(TSK *tskp)
{
	if(tskp->step >= 100) return;
	if(attract_mode != 0) return;

	switch(extend_step){
		case	0:
			extend_score = EXTEND_1ST;
			extend_step++;
		case	1:
			if(extend_score <= score){
				if(left < 4){
					SDL_PCB_SndPlaySe(SND_EXTEND);
					left++;
				}
				extend_step++;
				extend_score += EXTEND_2ND;
			}
			break;
		case	2:
			if(extend_score <= score){
				if(left < 4){
					SDL_PCB_SndPlaySe(SND_EXTEND);
					left++;
				}
				extend_score += EXTEND_3RD;
			}
			break;
	}
}

static void TSKshipTimeCtrl(TSK *tskp)
{
	if(!time_off){
		if(limit_time){
			limit_time--;
		}
	}
	if(limit_time && limit_time <= (60 * 11) && !(limit_time % 60)){
		SDL_PCB_SndPlaySe(SND_CNT00 + (limit_time / 60) - 1);
	}
}

static void TSKshipAttract(TSK *tskp)
{
	TSK *etskp;
	float dist = 1000.0f;
	float px, py;
	float dx, dy;

	ship_pads = PAD_A;

	px = tskp->pos[X];
	py = tskp->pos[Y];

	if(tskp->trg_tsk == NULL){
		for(etskp = TskIndex[GROUP_ENEMY][TSK_TOP].next_tsk; etskp != &(TskIndex[GROUP_ENEMY][TSK_BOTTOM]); etskp = etskp->next_tsk){
			if(etskp->step < 100 && etskp->step != -1){
				dx = fabs(etskp->pos[X] - px);
				dy = fabs(etskp->pos[Y] - py);
				if(dist > (dx + dy)){
					tskp->trg_tsk = etskp;
					dist = dx + dy;
				}
			}
		}
	}

	if(tskp->trg_tsk == NULL) return;

	dx = tskp->trg_tsk->pos[X];
	dy = tskp->trg_tsk->pos[Y];

	if(fabs(px - dx) > 4.0f){
		if(px < dx) ship_pads |= PAD_RI;
		else		ship_pads |= PAD_LF;
	}else if(fabs(py - dy) > 4.0f){
		if(py < dy) ship_pads |= PAD_DN;
		else		ship_pads |= PAD_UP;
	}else{
		if(fabs(px - dx) > fabs(py - dy)){
			if(px < dx) ship_pads |= PAD_RI;
			else		ship_pads |= PAD_LF;
		}else{
			if(py < dy) ship_pads |= PAD_DN;
			else		ship_pads |= PAD_UP;
		}
	}
}

static void ship_destroy(TSK *tskp)
{
	tskp->tskid |= TSKID_DESTROY;
	tskp->int_adr = NULL;
	if(left){
		if(!attract_mode) left--;
		tskp->wait = 60;
		tskp->step = 20;
	}else{
		tskp->step = 100;
	}
}
