#include "app.h"

#define PAL_R 240
#define PAL_G 160
#define PAL_B 160
#define ROLL_RATE (0.0713275f)

static void TSKzako05Int(TSK *tskp);
static void TSKzako05Exit(TSK *tskp);
static void TSKzako05Draw(TSK *tskp);
static void TSKzako05SetForm(TSK *tskp);
static int TSKzako05ShotCtrl(TSK *tskp);

static VERTEX ver_org[] = {
							{ fld(+0.0), fld(-16.0) },
							{ fld(+8.0), fld( -8.0) },
							{ fld(+8.0), fld( +8.0) },
							{ fld(+0.0), fld(+16.0) },
							{ fld(-8.0), fld( +8.0) },
							{ fld(-8.0), fld( -8.0) },
						};

static PAL32 color;

void TSKzako05(TSK *tskp)
{
	switch(tskp->step){
		case	0:
			tskp->tskid |= TSKID_ENEMY;
			tskp->int_adr = NULL;
			tskp->exit_adr = &TSKzako05Exit;
			tskp->draw_adr = &TSKzako05Draw;
			TSKzako05SetForm(tskp);
			TSKenemyPosSet(tskp);
			tskp->col[X] = 12.0f;
			tskp->col[Y] = 12.0f;
			tskp->roll = 0.0f;
			tskp->cnt = 0;
			tskp->score = 300;
			TSKpalleteSet(tskp, 0, 0, 0, 255);
			SDL_PCB_SndPlaySe(SND_ENEMY);
			tskp->step++;
			break;
		case	1:
			if(!tskp->cnt){
				tskp->cnt = 4;
				switch(rand() % 4){
					case 0: tskp->mode = PAD_RI; break;
					case 1: tskp->mode = PAD_LF; break;
					case 2: tskp->mode = PAD_DN; break;
					case 3: tskp->mode = PAD_UP; break;
				}
			}
			tskp->wait = 30;
			tskp->roll += ROLL_RATE;
			tskp->step++;
			break;
		case	2:
			if(tskp->wait){
				if(tskp->int_adr == NULL){
					if(tskp->color.r < PAL_R) tskp->color.r += 8;
					else					  tskp->color.r  = PAL_R;
					if(tskp->color.g < PAL_G) tskp->color.g += 8;
					else					  tskp->color.g  = PAL_G;
					if(tskp->color.b < PAL_B) tskp->color.b += 8;
					else					  tskp->color.b  = PAL_B;
					TSKpalleteSet(tskp, tskp->color.r, tskp->color.g, tskp->color.b, 255);
					if(tskp->color.r == PAL_R) tskp->int_adr = &TSKzako05Int;
				}else{
					if(tskp->wait < 30){
						color.r = rand() % 255;
						color.g = rand() % 255;
						color.b = rand() % 255;
						TSKpalleteSet(tskp, color.r, color.g, color.b, 255);
					}
				}
				tskp->wait--;
			}else{
				TSKpalleteSet(tskp, PAL_R, PAL_G, PAL_B, 255);
				if(TSKzako05ShotCtrl(tskp)){
					tskp->cnt--;
					if(tskp->cnt){
						tskp->step = 1;
					}else{
						tskp->wait = 60 - isTimeUp() * 30;
						tskp->step++;
					}
				}
			}
			tskp->roll += ROLL_RATE;
			break;
		case	3:
			if(tskp->wait){
				tskp->wait--;
			}else{
				tskp->step++;
			}
			tskp->roll += ROLL_RATE;
			break;
		case	4:
			if((rand() % 100) > 50){
				if((rand() % 100) > 50){
					tskp->vel[X] = -32.0f / 15.0f;
					tskp->vel[Y] = 0.0f;
				}else{
					tskp->vel[X] = +32.0f / 15.0f;
					tskp->vel[Y] = 0.0f;
				}
			}else{
				if((rand() % 100) > 50){
					tskp->vel[X] = 0.0f;
					tskp->vel[Y] = -32.0f / 15.0f;
				}else{
					tskp->vel[X] = 0.0f;
					tskp->vel[Y] = +32.0f / 15.0f;
				}
			}
			tskp->roll += ROLL_RATE;
			tskp->wait = 15;
			tskp->step++;
			break;
		case	5:
			if(tskp->wait){
				tskp->wait--;
				tskp->pos[X] += tskp->vel[X];
				tskp->pos[Y] += tskp->vel[Y];
			}else{
				tskp->step = 1;
			}
			TSKareaClamp(tskp);
			tskp->roll += ROLL_RATE;
			break;
		case	100:
			TSKpalleteSet(tskp, PAL_R, PAL_G, PAL_B, 255);
			SDL_PCB_SndPlaySe(SND_EDEST);
			enemy_destroy(tskp);
			setTSKbroken(tskp, 8, 4.0f);
			color.r = 255;
			color.g = 255;
			color.b = 0;
			color.a = 255;
			setTSKparticle(tskp, 8, 8.0f, color);
			tskp->step = -1;
			break;

		default:
			TSKclear(tskp);
			break;
	}
}

static void TSKzako05Int(TSK *tskp)
{
	if(tskp->tsk_collision->tskid & TSKID_SHIP) return;
	tskp->int_adr = NULL;
	tskp->score *= tskp->tsk_collision->level + 1;
	tskp->step = 100;
}

static void TSKzako05Exit(TSK *tskp)
{
}

static void TSKzako05Draw(TSK *tskp)
{
	drawGRPlineObj(tskp);
}

static void TSKzako05SetForm(TSK *tskp)
{
	setGRPlineObj(tskp, ver_org, ARRAY_SIZE(ver_org));
}

static int TSKzako05ShotCtrl(TSK *tskp)
{
	TSK *etskp;
	float len = ESHOT_LENGTH;

	if(getShipDist(tskp) < 32.0f) return 0;

	etskp = TSKsetBtm(&TSKeshot, GROUP_ESHOT);
	if(etskp){
		etskp->pos[X] = tskp->pos[X];
		etskp->pos[Y] = tskp->pos[Y];
		etskp->len = len;
		switch(tskp->mode){
			case	PAD_RI:
				etskp->vel[X] = +ESHOT_SPD + isTimeUp() * 2.0f;
				etskp->vel[Y] = 0.0f;
				etskp->roll = +PI / 2.0f;
				tskp->mode = PAD_DN;
				break;
			case	PAD_DN:
				etskp->vel[X] = 0.0f;
				etskp->vel[Y] = +ESHOT_SPD - isTimeUp() * 2.0f;
				etskp->roll = +PI;
				tskp->mode = PAD_LF;
				break;
			case	PAD_LF:
				etskp->vel[X] = -ESHOT_SPD + isTimeUp() * 2.0f;
				etskp->vel[Y] = 0.0f;
				etskp->roll = -PI / 2.0f;
				tskp->mode = PAD_UP;
				break;
			case	PAD_UP:
				etskp->vel[X] = 0.0f;
				etskp->vel[Y] = -ESHOT_SPD - isTimeUp() * 2.0f;
				etskp->roll = +0.0f;
				tskp->mode = PAD_RI;
				break;
		}
		TSKareaClamp(etskp);
	}

	return 1;
}
