#include "app.h"

static TSK *TskEmptyptr;

void TSKinit(void)
{
	int	i;

	for(i = 0; i < TSK_MAX - 1; i++){
		TskBuf[i].emptr = &(TskBuf[i+1]);
	}

	TskEmptyptr = TskBuf;
	TskBuf[TSK_MAX-1].emptr = NULL;

	for(i = 0; i < TSK_GROUP_MAX; i++){
		TskIndex[i][TSK_TOP].prev_tsk = NULL;
		TskIndex[i][TSK_TOP].next_tsk = &(TskIndex[i][TSK_BOTTOM]);
		TskIndex[i][TSK_BOTTOM].prev_tsk = &(TskIndex[i][TSK_TOP]);
		TskIndex[i][TSK_BOTTOM].next_tsk = NULL;
	}
}

TSK *TSKsetTop(FUNC exec_adr, int group)
{
	TSK *tskp;

	if(!TskEmptyptr){
		DIE();
	}
	tskp = TskEmptyptr;
	TskEmptyptr = tskp->emptr;
	memset(tskp,0,sizeof(TSK));
	tskp->prog_adr = exec_adr;
	tskp->prev_tsk = &(TskIndex[group][TSK_TOP]);
	tskp->next_tsk = TskIndex[group][TSK_TOP].next_tsk;
	tskp->prev_tsk->next_tsk = tskp;
	tskp->next_tsk->prev_tsk = tskp;
	tskp->exit_adr = NULL;
	tskp->draw_adr = NULL;
	tskp->int_adr = NULL;
	tskp->trg_tsk = NULL;
	return tskp;
}

TSK *TSKsetBtm(FUNC exec_adr, int group)
{
	TSK	*tskp;

	if(!TskEmptyptr){
		DIE();
	}
	tskp = TskEmptyptr;
	TskEmptyptr = tskp->emptr;
	memset(tskp,0,sizeof(TSK));
	tskp->prog_adr = exec_adr;
	tskp->prev_tsk = TskIndex[group][TSK_BOTTOM].prev_tsk;
	tskp->next_tsk = &(TskIndex[group][TSK_BOTTOM]);
	tskp->prev_tsk->next_tsk = tskp;
	tskp->next_tsk->prev_tsk = tskp;
	tskp->exit_adr = NULL;
	tskp->draw_adr = NULL;
	tskp->int_adr = NULL;
	tskp->trg_tsk = NULL;
	return tskp;
}

void TSKexec(void)
{
	TSK *tskp;
	int i;

	TskEntCnt = 0;

	for(i = 0; i < TSK_GROUP_MAX; i++){
		for(tskp = TskIndex[i][TSK_TOP].next_tsk; tskp != &(TskIndex[i][TSK_BOTTOM]); tskp = tskp->next_tsk){
			++TskEntCnt;
			if(tskp->prog_adr) tskp->prog_adr(tskp);
		}
	}
}

void TSKdraw(void)
{
	TSK *tskp;
	int i;

	for(i = 0; i < TSK_GROUP_MAX; i++){
		for(tskp = TskIndex[i][TSK_TOP].next_tsk; tskp != &(TskIndex[i][TSK_BOTTOM]); tskp = tskp->next_tsk){
			if(tskp->draw_adr) tskp->draw_adr(tskp);
		}
	}
}

void TSKintExec(TSK *tskp)
{
	if(tskp->int_adr) tskp->int_adr(tskp);
}

void TSKclear(TSK *tskp)
{
	TSK *prev;
	TSK *next;

	if(tskp->exit_adr) tskp->exit_adr(tskp);

	prev = tskp->next_tsk->prev_tsk = tskp->prev_tsk;
	next = tskp->prev_tsk->next_tsk = tskp->next_tsk;

	memset(tskp,0,sizeof(TSK));

	tskp->emptr = TskEmptyptr;
	TskEmptyptr = tskp;

	tskp->prev_tsk = prev;
	tskp->next_tsk = next;
}

void TSKclearAll(TSK *tskp)
{
	TSK *etskp;
	TSK *ntskp;
	int i;

	for(i = 0; i < TSK_GROUP_MAX; i++){
		for(etskp = TskIndex[i][TSK_TOP].next_tsk; etskp != &(TskIndex[i][TSK_BOTTOM]); etskp = ntskp){
			ntskp = etskp->next_tsk;
			if(etskp != tskp) TSKclear(etskp);
		}
	}
}

void TSKclearGroup(TSK *tskp, int group)
{
	TSK *etskp;
	TSK *ntskp;

	for(etskp = TskIndex[group][TSK_TOP].next_tsk; etskp != &(TskIndex[group][TSK_BOTTOM]); etskp = ntskp){
		ntskp = etskp->next_tsk;
		if(etskp != tskp) TSKclear(etskp);
	}
}

